// PlayerKernel.h

// Jadro hraca, zakladne funkcie pre jeho funkcionalitu

#ifndef PLAYERKERNEL_H
#define PLAYERKERNEL_H

//#include "Player/Interfaces/Synchronization.h"
#include "Player/Interfaces/Memory.h"
#include "Player/Interfaces/Tactics.h"
//#include "Player/World/World.h"
//#include "Common/Module.h"
#include "Common/Main.h"
#include "ModuleManager/DataStorage.h"
#include "ModuleManager/EventHandlers.h"
#include "ModuleManager/Behaviours/ActBehaviour.h"
#include "ModuleManager/Behaviours/TurnNeckToBehaviour.h"
#include "ModuleManager/Behaviours/ActTestBehaviour.h"

struct FEEDBACK_DATA;

// Jadro hraca so synchronizaciou a rozhodovanim
class PlayerKernel : public Module, public SynchronizationInterface, public MemoryInterface, public TacticsInterface
{
	USE_MODULE(ActBehaviour)
	USE_MODULE(TurnNeckToBehaviour)
	USE_MODULE(EventHandlers)
	USE_MODULE(ActTestBehaviour)

	// Sem sa ukladaju vsetky vykonane prikazy. Potom sa aplikuju tieto prikazy na model sveta,
	// tak aby odrazal, to co hrac vykonal.
	FEEDBACK_DATA* feedback;

	// Pocet spracovanych prikazov
	COMMAND_COUNTS* counts;

	// Synchronizacia
	SynchroMode synchMode;
	//SynchroMode nextSynchMode;
	bool decisionWasMade;

protected:	

	// Funkcia pre odstranenie informacii o lopte do nSteps cyklov
	void ClearPredictedBallStates(int nSteps);
	// Funkcia pre predpovedanie informacii o lopte v nasledujucich nSteps cykloch
	void PredictBallStates(Ball* actBallInfo, int nSteps);

	// Posledne sense_body a posledna vizualna informacia
	const SENSE_BODY* lastSb;
	const VISUAL_INFO* lastVi;
	
	int lastSbTime;
	int lastViTime;

	// Rozhodovanie sa
	virtual void Act()
	{
	
	}
	
public:
	// Strana hraca
	RealSide myside;
	
	// Konstruktor
	PlayerKernel(const char* name = "PlayerKernel", const char* desc = "PlayerKernel module");
	
	// Destruktor
	virtual ~PlayerKernel();

	// Implementacia rozhrania
	virtual Interface* GetInterface(int iTask) const;

	// Vracia pointer na model sveta
	virtual World* GetWorld() const
	{ 
		return world;
	}

	// Funkcie k synchroznizacnemu modu	
	virtual SynchroMode GetCurrentSynchMode() const
	{
		return synchMode;
	}
	virtual SynchroMode GetNextSynchMode() const
	{
		return DATA_STORAGE.GetNextSynchMode();
	}
	virtual void SetNextSynchMode(SynchroMode mode)
	{
		DATA_STORAGE.SetNextSynchMode(mode);
	}

	// Vnimanie a model sveta
	
	// Pri prichode spravy hear
	virtual void OnHear(int time, float direction, bool ourTeam, int uniform, const char* msg);
	// Pri zmene herneho modu
	virtual void OnChangeMode(int time, PlayMode mode);
	// Pri zmene typu hraca
	virtual void OnChangePlayerType(int uniform, int type);
	// Pri prichode spravy init
	virtual void OnInit(RealSide realSide, int uniform, PlayMode mode);
	// Handler pre informacie o heterogennych typoch hracov
	virtual void OnPlayerParam(const PLAYER_PARAMS* params);
	// Handler pre informacie o jednom heterogennom (alebo normalnom) type hraca
	virtual void OnPlayerType(int id, const PLAYER_TYPE* type);
	// Pri prichode spravy s informaciou o skore
	virtual void OnScore(int time, int our, int their);
	// Pri prichode zrakovej spravy see
	virtual void OnSee(const VISUAL_INFO*);
	// Pri prichode spravy sense_body
	virtual void OnSenseBody(const SENSE_BODY*);
	// Handler pre zmenu parametrov servera
	virtual void OnServerParam(const SERVER_PARAMS* params);
	
	virtual void OnFreeform(const char* msg);
//  virtual void OnSetParameterUseNN(const char* msg);	
	
	// Handlery pre ulozenie posielanych prikazov na server
	virtual void OnExecDash(float power);
	virtual void OnExecKick(float power, float direction);
	virtual void OnExecNeck(int angle);
	virtual void OnExecTurn(float moment);

	//pretazena metoda z tacticsinterface, zmeni formaciu
	virtual void ChangeFormation(const char* formation);
};

// Struktura pre ulozenie vykonanych prikazov
struct FEEDBACK_DATA
{
	float lastPower;
	float lastTurn;
	int lastNeck;
	float lastKickPower;
	float lastKickDir;
};

#endif // PLAYERKERNEL_H
